require 'date'
class GraphController < ApplicationController
  
  class GruffGraff
  
    def initialize(options)
      size = "700x400"
      @graph = options[:graph_type].new(size)

      @graph.theme = { :background_colors => ['#73BF26', '#ffffff'] }
      @graph.font = File.expand_path('/usr/X11R6/lib/X11/fonts/TTF/Vera.ttf',
                                   RAILS_ROOT)
      
      # fill in the data with the optional data name
      #Check to see if multiple datasets, if so, fill them all!     
      if options[:data].size > 1 && options[:data].all?  {|i| i.is_a?(Array)}
        options[:data].each do |array|
          @graph.data( options.fetch(:data_name, "Data"), array)
        end
      else #one dimensional array, just pass it in
      @graph.data( options.fetch(:data_name, "Data"), options[:data] )
      end
      
      # set the labels or create an empty hash
      @graph.labels = options[:interval_labels] \
        if options.has_key?(:interval_labels) and \
           options[:interval_labels].class == Hash
      @graph.x_axis_label = options[:x_axis_label] \
        if options.has_key?(:x_axis_label)
      @graph.y_axis_label = options[:y_axis_label] \
        if options.has_key?(:y_axis_label)
      @graph.title = options[:title] if options.has_key?(:title)
      
      @graph.minimum_value = 0.0

    end

    def output
      return([@graph.to_blob, {:disposition => 'inline', :type => 'image/png'}])
    end

  end

  # produce a graph of votes per day during an election
  def votes_per_day
    @election = Election.find(params[:id])
    data, labels = get_votes_per_day_data(@election)
    
    graph = GruffGraff.new( :graph_type => Gruff::Line,
                            :data_name => @election.name,
                            :data => data,
                            :interval_labels => labels,
                            :title => "Voters Per Day",
                            :x_axis_label => "Data",
                            :y_axis_label =>"Number of Votes")
    send_data(*graph.output)
  end
  
  #will place votes in a fixed number of intervals, and shows votes over time
  def votes_per_interval
    @election = Election.find(params[:id])
    data, labels, scale = get_votes_per_interval_data(@election)
    
    graph = GruffGraff.new( :graph_type => Gruff::Line,
                            :data_name => @election.name,
                            :data => data,
                            :interval_labels => labels,
                            :title => "Voters Over Time",
                            :x_axis_label => scale,
                            :y_axis_label => "Number of Votes")
    send_data(*graph.output)
  end

  def borda_bar
    @election = Election.find(params[:id])
    pref_tally = make_preference_tally(@election)
    
    @borda_result = BordaVote.new(pref_tally).result
    data, labels = get_borda_points(@borda_result)
    
    graph = GruffGraff.new( :graph_type => Gruff::Bar,
                            :data_name => @election.name,
                            :data => data,
                            :interval_labels => labels,
                            :title => "Points Per Candidate",
                            :y_axis_label => "Points",
                            :x_axis_label => "Candidate")
    send_data(*graph.output)
  end
 
  def choices_positions
    @election = Election.find(params[:id])
    pref_tally = make_preference_tally(@election)
    
    fulldata, labels = get_positions_info(@election)
    labels = @candidates
    graph = GruffGraff.new( :graph_type => Gruff::Bar,
                            :data_name => @election.name,
                            :data => fulldata,
                            :interval_labels => labels,
                            :title => "Times Voted in Each Position",
                            :y_axis_label => "Number of Times Ranked",
                            :x_axis_label => "Rank")
    send_data(*graph.output) 
  end
  
 private 
 
  def get_positions_info(election)
    buckets = Hash.new
    buckets2= Hash.new
    rank_labels = Hash.new
  
    election.candidates.each do |candidate|
      buckets[candidate.id] = []
      buckets2[candidate.id] = []
    end
     
    election.votes.each do |vote|
      vote.rankings.each do |ranking|
        buckets[ranking.candidate_id] << ranking.rank
      end
    end
       
    buckets.each_pair do |id, array|
      (1..election.candidates.size).each do |i|
        buckets2[id] << (array.find_all {|rank| rank == i}).size
      end
    end
    
    election.votes.each do |vote|
      vote.rankings.size.times do |i|
        rank_labels[i] = (i+1).to_s
      end
    end
   
    return buckets2.values, rank_labels
    
  end
   
  # generate the data and labels for each graph
  def get_votes_per_day_data(election)
    voter_days = Array.new
    unique_days = Array.new
    total_per_day = Array.new
    election_days = Hash.new
    
    #turn election startdate into date object, and create the range of election
    startdate = Date.parse(election.startdate.to_s)
    election_range = startdate..Date.today
    
    # create a hash with all the dates of the election in String format
    # referenced by their order in the election
    election_range.each_with_index do |day, index|
      election_days[index] = day.to_s
    end
    
    # Now I need to create an array with all the times votes were made
    election.votes.each do |vote|
        voter_days << Date.parse(vote.time.to_s)
    end
    voter_days.sort!
    
    # Now I need to count how many times each each date appears in voter_days,
    # and put that number into a votes_per_day array, the 'data' for the graph    
    #Create an array of unique days from voter_days
    voter_days.each do |day|
      unless unique_days.any? {|date| date.eql?(day)}
        unique_days << day
      end
    end
    unique_days.sort!
    
    #find all dates where those days = date at current index, put size of returned
    #array into total_per_day
    unique_days.each_with_index do |date, index|
      total_per_day << (voter_days.select {|day| day.eql?(date)}).size
    end    

    # return the data and the labels
    return total_per_day, election_days
   
  end
  
  def get_votes_per_interval_data(election)
    labels_hash = Hash.new
    buckets = Hash.new
    total_per_interval = Array.new
    interval_type = ""
    
    starttime = election.startdate
    timedelta = Time.now - starttime
    numcols = 10
    interval_length = timedelta/numcols
    
    # Make a hash, buckets, indexed by time intervals and containing empty arrays
    # The time object must come first in addition! 
    # i would start at 0, i+1 goes from 1 up till numcols
    numcols.times {|i| buckets[starttime + ((i+1)*interval_length)] = []}
     
    # Put votes into bucket according to the time interval to which they belong,
    # referenced by their key
    # Will build a graph over time, as each successive interval will have more
    # vote objects  
    election.votes.each do |vote|
      buckets.keys.sort.each do |inter|
        if vote.time < inter
          buckets[inter] << vote
        end
      end
    end
  
    total_per_interval = buckets.keys.sort.collect {|key| buckets[key].size}
    
    # Create the hash for the labels. Each graph has ten columns, and three
    # will be labeled
    if timedelta < 2.hours #under two hours use minutes for labels
      labels_hash[0] = starttime.min.to_s
      labels_hash[(numcols/2)-1] = (starttime + (timedelta/2)).min.to_s
      labels_hash[numcols-1] = Time.now.min.to_s
      interval_type = "Minute of the Hour"
    elsif timedelta < 2.days #more than 2 hours means use hours for labels
      labels_hash[0] = starttime.hour.to_s
      labels_hash[(numcols/2)-1] = (starttime + (timedelta/2)).hour.to_s
      labels_hash[numcols-1] = Time.now.hour.to_s
      interval_type = "Hour of the Day on 24 hour scale"
    else #more than 2 days means use dates for labels
      labels_hash[0] = (Date.parse(starttime.to_s)).to_s
      labels_hash[(numcols/2)-1] = (Date.parse((starttime + (timedelta/2)).to_s)).to_s
      labels_hash[numcols-1] = (Date.today).to_s
      interval_type = "The Date"
    end
    
    # Make sure to return an array for data and hash for labels
    return total_per_interval, labels_hash, interval_type   
  end
  
  def get_borda_points(result)
    #points holds how mnay points each candidate has received in array form
    #becasue Gruff::Bar#data takes only an array
    points = Array.new
    labels = Hash.new

    #Populate points with an sorted array from election.votes hash
    #biggest to smallest will go from left to right
    points = result.election.votes.sort do |a, b|
      b[1] <=> a[1]
    end.collect {|i| i[1]}

    #make the labels  
    result.ranked_candidates.each_with_index do |candidate, index|
      labels[index] = Candidate.find(candidate).name
    end

    return points, labels
  end

  #most vote result objects require an array of vote arrays, which this will make
  def make_preference_tally(election)
    preference_tally = Array.new
    @election.voters.each do |voter|
      next unless voter.voted?
      preference_tally << voter.vote.rankings.sort.collect \
        { |ranking| ranking.candidate.id }
    end
  return preference_tally
  end

end
